@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -32,6 +32,10 @@
 #include "results.h"
 
 
+/*+ To help when debugging +*/
+#define DEBUG 0
+
+
 /* Global variables */
 
 /*+ The option not to print any progress information. +*/
@@ -79,6 +83,10 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
  Result  *result1,*result2;
  int     force_uturn=0;
 
+#if DEBUG
+ printf("  FindNormalRoute(...,start_node=%"Pindex_t" prev_segment=%"Pindex_t" finish_node=%"Pindex_t")\n",start_node,prev_segment,finish_node);
+#endif
+
  /* Set up the finish conditions */
 
  finish_score=INF_SCORE;
@@ -297,12 +305,27 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
  if(!finish_result)
    {
+#if DEBUG
+    printf("    Failed\n");
+#endif
+
     FreeResultsList(results);
     return(NULL);
    }
 
  FixForwardRoute(results,finish_result);
 
+#if DEBUG
+ Result *r=FindResult(results,results->start_node,results->prev_segment);
+
+ while(r)
+   {
+    printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+    r=r->next;
+   }
+#endif
+
  return(results);
 }
 
@@ -337,8 +360,14 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
  Result  *result1,*result2,*result3,*result4;
  int     force_uturn=0;
 
+#if DEBUG
+ printf("  FindMiddleRoute(...,[begin has %d nodes],[end has %d nodes])\n",begin->number,end->number);
+#endif
+
+#if !DEBUG
  if(!option_quiet)
     printf_first("Routing: Super-Nodes checked = 0");
+#endif
 
  /* Set up the finish conditions */
 
@@ -584,8 +613,10 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
              InsertInQueue(queue,result2);
          }
 
+#if !DEBUG
        if(!option_quiet && !(results->number%1000))
           printf_middle("Routing: Super-Nodes checked = %d",results->number);
+#endif
 
       endloop:
 
@@ -593,8 +624,10 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
       }
    }
 
+#if !DEBUG
  if(!option_quiet)
     printf_last("Routing: Super-Nodes checked = %d",results->number);
+#endif
 
  FreeQueueList(queue);
 
@@ -602,6 +635,10 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
  if(!finish_result)
    {
+#if DEBUG
+    printf("    Failed\n");
+#endif
+
     FreeResultsList(results);
     return(NULL);
    }
@@ -620,6 +657,17 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
  FixForwardRoute(results,finish_result);
 
+#if DEBUG
+ Result *r=FindResult(results,results->start_node,results->prev_segment);
+
+ while(r)
+   {
+    printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+    r=r->next;
+   }
+#endif
+
  return(results);
 }
 
@@ -719,6 +767,10 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
  Queue   *queue;
  Result  *result1,*result2;
 
+#if DEBUG
+ printf("    FindSuperRoute(...,start_node=%"Pindex_t" finish_node=%"Pindex_t")\n",start_node,finish_node);
+#endif
+
  /* Create the list of results and insert the first node into the queue */
 
  results=NewResultsList(64);
@@ -809,6 +861,17 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
 
  FreeQueueList(queue);
 
+#if DEBUG
+ Result *r=FindResult(results,results->start_node,results->prev_segment);
+
+ while(r)
+   {
+    printf("      node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+    r=r->next;
+   }
+#endif
+
  return(results);
 }
 
@@ -833,16 +896,18 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
   index_t prev_segment The previous segment before the start node.
 
   index_t finish_node The finish node.
-
-  int *nsuper Returns the number of super-nodes seen.
   ++++++++++++++++++++++++++++++++++++++*/
 
-Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t start_node,index_t prev_segment,index_t finish_node,int *nsuper)
+Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t start_node,index_t prev_segment,index_t finish_node)
 {
  Results *results;
  Queue   *queue;
  Result  *result1,*result2;
- int     found_finish=0,force_uturn=0;
+ int     nsuper=0,force_uturn=0;
+
+#if DEBUG
+ printf("  FindStartRoutes(...,start_node=%"Pindex_t" prev_segment=%"Pindex_t" finish_node=%"Pindex_t")\n",start_node,prev_segment,finish_node);
+#endif
 
  /* Create the list of results and insert the first node into the queue */
 
@@ -999,10 +1064,10 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
           result2->score=cumulative_score;
 
           if(node2p && IsSuperNode(node2p))
-             (*nsuper)++;
+             nsuper++;
 
           if(node2==finish_node)
-             found_finish=1;
+             nsuper++;
          }
        else if(cumulative_score<result2->score) /* New score for end node/segment combination is better */
          {
@@ -1038,12 +1103,302 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
  /* Check it worked */
 
- if(results->number==1 || (*nsuper==0 && found_finish==0))
+ if(results->number==1 || nsuper==0)
    {
+#if DEBUG
+    printf("    Failed (%d results, %d super)\n",results->number,nsuper);
+#endif
+
     FreeResultsList(results);
     return(NULL);
    }
 
+#if DEBUG
+ Result *s=FirstResult(results);
+
+ while(s)
+   {
+    if(s->node==finish_node)
+      {
+       Result *r=FindResult(results,s->node,s->segment);
+
+       printf("    -------- route to finish node\n");
+
+       while(r)
+         {
+          printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+          r=r->prev;
+         }
+      }
+
+    if(!IsFakeNode(s->node))
+      {
+       Node *n=LookupNode(nodes,s->node,1);
+
+       if(IsSuperNode(n))
+         {
+          Result *r=FindResult(results,s->node,s->segment);
+
+          printf("    -------- route to super node\n");
+
+          while(r)
+            {
+             printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+             r=r->prev;
+            }
+         }
+      }
+
+    s=NextResult(results,s);
+   }
+#endif
+
+ return(results);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Continue finding routes from a set of super-nodes to a finish point.
+
+  Results *ExtendStartRoutes Returns the set of results that were passed in.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  Profile *profile The profile containing the transport type, speeds and allowed highways.
+
+  Results *being The partial set of routes already computed.
+
+  index_t finish_node The finish node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,Results *begin,index_t finish_node)
+{
+ Results *results=begin;
+ Queue   *queue;
+ Result  *result1,*result2,*result3;
+ Result  *finish_result=NULL;
+ score_t finish_score=INF_SCORE;
+
+#if DEBUG
+ printf("  ExtendStartRoutes(...,[begin has %d nodes],finish_node=%"Pindex_t")\n",begin->number,finish_node);
+#endif
+
+ /* Check the list of results and insert the super nodes into the queue */
+
+ queue=NewQueueList();
+
+ result3=FirstResult(begin);
+
+ while(result3)
+   {
+    if(result3->node==finish_node)
+       if(result3->score<finish_score)
+         {
+          finish_score=result3->score;
+          finish_result=result3;
+         }
+
+    if(!IsFakeNode(result3->node))
+       if(IsSuperNode(LookupNode(nodes,result3->node,5)))
+          InsertInQueue(queue,result3);
+
+    result3=NextResult(results,result3);
+   }
+
+ /* Loop across all nodes in the queue */
+
+ while((result1=PopFromQueue(queue)))
+   {
+    Node *node1p=NULL;
+    Segment *segmentp;
+    index_t node1,seg1,seg1r;
+    index_t turnrelation=NO_RELATION;
+
+    /* score must be better than current best score */
+    if(result1->score>=finish_score)
+       continue;
+
+    node1=result1->node;
+    seg1=result1->segment;
+
+    if(IsFakeSegment(seg1))
+       seg1r=IndexRealSegment(seg1);
+    else
+       seg1r=seg1;
+
+    if(!IsFakeNode(node1))
+       node1p=LookupNode(nodes,node1,1);
+
+    /* lookup if a turn restriction applies */
+    if(profile->turns && node1p && IsTurnRestrictedNode(node1p))
+       turnrelation=FindFirstTurnRelation2(relations,node1,seg1r);
+
+    /* Loop across all segments */
+
+    if(IsFakeNode(node1))
+       segmentp=FirstFakeSegment(node1);
+    else
+       segmentp=FirstSegment(segments,node1p,1);
+
+    while(segmentp)
+      {
+       Node *node2p=NULL;
+       Way *wayp;
+       index_t node2,seg2,seg2r;
+       score_t segment_pref,segment_score,cumulative_score;
+       int i;
+
+       node2=OtherNode(segmentp,node1); /* need this here because we use node2 at the end of the loop */
+
+       /* must be a normal segment */
+       if(!IsNormalSegment(segmentp))
+          goto endloop;
+
+       /* must obey one-way restrictions (unless profile allows) */
+       if(profile->oneway && IsOnewayTo(segmentp,node1))
+          goto endloop;
+
+       if(IsFakeNode(node1) || IsFakeNode(node2))
+         {
+          seg2 =IndexFakeSegment(segmentp);
+          seg2r=IndexRealSegment(seg2);
+         }
+       else
+         {
+          seg2 =IndexSegment(segments,segmentp);
+          seg2r=seg2;
+         }
+
+       /* must not perform U-turn (unless profile allows) */
+       if(profile->turns && (seg1==seg2 || seg1==seg2r || seg1r==seg2 || (seg1r==seg2r && IsFakeUTurn(seg1,seg2))))
+          goto endloop;
+
+       /* must obey turn relations */
+       if(turnrelation!=NO_RELATION && !IsTurnAllowed(relations,turnrelation,node1,seg1r,seg2r,profile->allow))
+          goto endloop;
+
+       wayp=LookupWay(ways,segmentp->way,1);
+
+       /* mode of transport must be allowed on the highway */
+       if(!(wayp->allow&profile->allow))
+          goto endloop;
+
+       /* must obey weight restriction (if exists) */
+       if(wayp->weight && wayp->weight<profile->weight)
+          goto endloop;
+
+       /* must obey height/width/length restriction (if exists) */
+       if((wayp->height && wayp->height<profile->height) ||
+          (wayp->width  && wayp->width <profile->width ) ||
+          (wayp->length && wayp->length<profile->length))
+          goto endloop;
+
+       segment_pref=profile->highway[HIGHWAY(wayp->type)];
+
+       /* highway preferences must allow this highway */
+       if(segment_pref==0)
+          goto endloop;
+
+       for(i=1;i<Property_Count;i++)
+          if(ways->file.props & PROPERTIES(i))
+            {
+             if(wayp->props & PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
+
+       /* profile preferences must allow this highway */
+       if(segment_pref==0)
+          goto endloop;
+
+       if(!IsFakeNode(node2))
+          node2p=LookupNode(nodes,node2,2);
+
+       /* mode of transport must be allowed through node2 unless it is the final node */
+       if(node2p && node2!=finish_node && !(node2p->allow&profile->allow))
+          goto endloop;
+
+       if(option_quickest==0)
+          segment_score=(score_t)DISTANCE(segmentp->distance)/segment_pref;
+       else
+          segment_score=(score_t)Duration(segmentp,wayp,profile)/segment_pref;
+
+       cumulative_score=result1->score+segment_score;
+
+       /* score must be better than current best score */
+       if(cumulative_score>=finish_score)
+          goto endloop;
+
+       result2=FindResult(results,node2,seg2);
+
+       if(!result2) /* New end node/segment combination */
+         {
+          result2=InsertResult(results,node2,seg2);
+          result2->prev=result1;
+          result2->score=cumulative_score;
+         }
+       else if(cumulative_score<result2->score) /* New score for end node/segment combination is better */
+         {
+          result2->prev=result1;
+          result2->score=cumulative_score;
+         }
+       else
+          goto endloop;
+
+       if(node2==finish_node)
+         {
+          if(cumulative_score<finish_score)
+            {
+             finish_score=cumulative_score;
+             finish_result=result2;
+            }
+         }
+       else
+         {
+          result2->sortby=result2->score;
+          InsertInQueue(queue,result2);
+         }
+
+      endloop:
+
+       if(IsFakeNode(node1))
+          segmentp=NextFakeSegment(segmentp,node1);
+       else if(IsFakeNode(node2))
+          segmentp=NULL; /* cannot call NextSegment() with a fake segment */
+       else
+         {
+          segmentp=NextSegment(segments,segmentp,node1);
+
+          if(!segmentp && IsFakeNode(finish_node))
+             segmentp=ExtraFakeSegment(node1,finish_node);
+         }
+      }
+   }
+
+ FreeQueueList(queue);
+
+ FixForwardRoute(results,finish_result);
+
+#if DEBUG
+ Result *r=FindResult(results,results->start_node,results->prev_segment);
+
+ while(r)
+   {
+    printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+    r=r->next;
+   }
+#endif
+
  return(results);
 }
 
@@ -1072,6 +1427,10 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
  Queue   *queue;
  Result  *result1,*result2,*result3;
 
+#if DEBUG
+ printf("  FindFinishRoutes(...,finish_node=%"Pindex_t")\n",finish_node);
+#endif
+
  /* Create the results and insert the finish node into the queue */
 
  results=NewResultsList(64);
@@ -1246,6 +1605,10 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
 
  if(results->number==1)
    {
+#if DEBUG
+    printf("    Failed\n");
+#endif
+
     FreeResultsList(results);
     return(NULL);
    }
@@ -1289,6 +1652,34 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
 
  FreeResultsList(results);
 
+#if DEBUG
+ Result *s=FirstResult(results2);
+
+ while(s)
+   {
+    if(!IsFakeNode(s->node))
+      {
+       Node *n=LookupNode(nodes,s->node,1);
+
+       if(IsSuperNode(n))
+         {
+          Result *r=FindResult(results2,s->node,s->segment);
+
+          printf("    --------\n");
+
+          while(r)
+            {
+             printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+             r=r->next;
+            }
+         }
+      }
+
+    s=NextResult(results2,s);
+   }
+#endif
+
  return(results2);
 }
 
@@ -1318,6 +1709,10 @@ Results *CombineRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
  Result *midres,*comres1;
  Results *combined;
 
+#if DEBUG
+ printf("  CombineRoutes(...,[begin has %d nodes],[middle has %d nodes])\n",begin->number,middle->number);
+#endif
+
  combined=NewResultsList(256);
 
  combined->start_node=begin->start_node;
@@ -1354,7 +1749,7 @@ Results *CombineRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
 
        comres2=InsertResult(combined,begres->node,begres->segment);
 
-       comres2->score=begres->score+comres1->score;
+       comres2->score=begres->score;
        comres2->prev=comres1;
 
        begres=begres->next;
@@ -1401,7 +1796,7 @@ Results *CombineRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
 
           comres2=InsertResult(combined,result->node,result->segment);
 
-          comres2->score=result->score+comres1->score;
+          comres2->score=midres->score+result->score;
           comres2->prev=comres1;
 
           result=result->next;
@@ -1419,6 +1814,17 @@ Results *CombineRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
 
  FixForwardRoute(combined,comres1);
 
+#if DEBUG
+ Result *r=FindResult(combined,combined->start_node,combined->prev_segment);
+
+ while(r)
+   {
+    printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+    r=r->next;
+   }
+#endif
+
  return(combined);
 }
 
