@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -30,7 +30,6 @@
 
 #include "files.h"
 #include "profiles.h"
-#include "functions.h"
 #include "xmlparse.h"
 
 
@@ -48,30 +47,67 @@ static int nloaded_profiles=0;
 //static int xmlDeclaration_function(const char *_tag_,int _type_,const char *version,const char *encoding);
 //static int RoutinoProfilesType_function(const char *_tag_,int _type_);
 static int profileType_function(const char *_tag_,int _type_,const char *name,const char *transport);
-//static int restrictionsType_function(const char *_tag_,int _type_);
-static int lengthType_function(const char *_tag_,int _type_,const char *limit);
-static int widthType_function(const char *_tag_,int _type_,const char *limit);
-static int heightType_function(const char *_tag_,int _type_,const char *limit);
-static int weightType_function(const char *_tag_,int _type_,const char *limit);
-static int turnsType_function(const char *_tag_,int _type_,const char *obey);
-//static int propertiesType_function(const char *_tag_,int _type_);
-static int onewayType_function(const char *_tag_,int _type_,const char *obey);
-static int propertyType_function(const char *_tag_,int _type_,const char *type,const char *percent);
-//static int preferencesType_function(const char *_tag_,int _type_);
-static int preferenceType_function(const char *_tag_,int _type_,const char *highway,const char *percent);
 //static int speedsType_function(const char *_tag_,int _type_);
+//static int preferencesType_function(const char *_tag_,int _type_);
+//static int propertiesType_function(const char *_tag_,int _type_);
+//static int restrictionsType_function(const char *_tag_,int _type_);
 static int speedType_function(const char *_tag_,int _type_,const char *highway,const char *kph);
+static int preferenceType_function(const char *_tag_,int _type_,const char *highway,const char *percent);
+static int propertyType_function(const char *_tag_,int _type_,const char *type,const char *percent);
+static int onewayType_function(const char *_tag_,int _type_,const char *obey);
+static int turnsType_function(const char *_tag_,int _type_,const char *obey);
+static int weightType_function(const char *_tag_,int _type_,const char *limit);
+static int heightType_function(const char *_tag_,int _type_,const char *limit);
+static int widthType_function(const char *_tag_,int _type_,const char *limit);
+static int lengthType_function(const char *_tag_,int _type_,const char *limit);
 
 
-/* The XML tag definitions */
+/* The XML tag definitions (forward declarations) */
 
-/*+ The speedType type tag. +*/
-static xmltag speedType_tag=
-              {"speed",
-               2, {"highway","kph"},
-               speedType_function,
+static xmltag xmlDeclaration_tag;
+static xmltag RoutinoProfilesType_tag;
+static xmltag profileType_tag;
+static xmltag speedsType_tag;
+static xmltag preferencesType_tag;
+static xmltag propertiesType_tag;
+static xmltag restrictionsType_tag;
+static xmltag speedType_tag;
+static xmltag preferenceType_tag;
+static xmltag propertyType_tag;
+static xmltag onewayType_tag;
+static xmltag turnsType_tag;
+static xmltag weightType_tag;
+static xmltag heightType_tag;
+static xmltag widthType_tag;
+static xmltag lengthType_tag;
+
+
+/* The XML tag definition values */
+
+/*+ The complete set of tags at the top level. +*/
+static xmltag *xml_toplevel_tags[]={&xmlDeclaration_tag,&RoutinoProfilesType_tag,NULL};
+
+/*+ The xmlDeclaration type tag. +*/
+static xmltag xmlDeclaration_tag=
+              {"xml",
+               2, {"version","encoding"},
+               NULL,
                {NULL}};
 
+/*+ The RoutinoProfilesType type tag. +*/
+static xmltag RoutinoProfilesType_tag=
+              {"routino-profiles",
+               0, {NULL},
+               NULL,
+               {&profileType_tag,NULL}};
+
+/*+ The profileType type tag. +*/
+static xmltag profileType_tag=
+              {"profile",
+               2, {"name","transport"},
+               profileType_function,
+               {&speedsType_tag,&preferencesType_tag,&propertiesType_tag,&restrictionsType_tag,NULL}};
+
 /*+ The speedsType type tag. +*/
 static xmltag speedsType_tag=
               {"speeds",
@@ -79,13 +115,6 @@ static xmltag speedsType_tag=
                NULL,
                {&speedType_tag,NULL}};
 
-/*+ The preferenceType type tag. +*/
-static xmltag preferenceType_tag=
-              {"preference",
-               2, {"highway","percent"},
-               preferenceType_function,
-               {NULL}};
-
 /*+ The preferencesType type tag. +*/
 static xmltag preferencesType_tag=
               {"preferences",
@@ -93,6 +122,34 @@ static xmltag preferencesType_tag=
                NULL,
                {&preferenceType_tag,NULL}};
 
+/*+ The propertiesType type tag. +*/
+static xmltag propertiesType_tag=
+              {"properties",
+               0, {NULL},
+               NULL,
+               {&propertyType_tag,NULL}};
+
+/*+ The restrictionsType type tag. +*/
+static xmltag restrictionsType_tag=
+              {"restrictions",
+               0, {NULL},
+               NULL,
+               {&onewayType_tag,&turnsType_tag,&weightType_tag,&heightType_tag,&widthType_tag,&lengthType_tag,NULL}};
+
+/*+ The speedType type tag. +*/
+static xmltag speedType_tag=
+              {"speed",
+               2, {"highway","kph"},
+               speedType_function,
+               {NULL}};
+
+/*+ The preferenceType type tag. +*/
+static xmltag preferenceType_tag=
+              {"preference",
+               2, {"highway","percent"},
+               preferenceType_function,
+               {NULL}};
+
 /*+ The propertyType type tag. +*/
 static xmltag propertyType_tag=
               {"property",
@@ -107,13 +164,6 @@ static xmltag onewayType_tag=
                onewayType_function,
                {NULL}};
 
-/*+ The propertiesType type tag. +*/
-static xmltag propertiesType_tag=
-              {"properties",
-               0, {NULL},
-               NULL,
-               {&propertyType_tag,NULL}};
-
 /*+ The turnsType type tag. +*/
 static xmltag turnsType_tag=
               {"turns",
@@ -149,374 +199,41 @@ static xmltag lengthType_tag=
                lengthType_function,
                {NULL}};
 
-/*+ The restrictionsType type tag. +*/
-static xmltag restrictionsType_tag=
-              {"restrictions",
-               0, {NULL},
-               NULL,
-               {&onewayType_tag,&turnsType_tag,&weightType_tag,&heightType_tag,&widthType_tag,&lengthType_tag,NULL}};
-
-/*+ The profileType type tag. +*/
-static xmltag profileType_tag=
-              {"profile",
-               2, {"name","transport"},
-               profileType_function,
-               {&speedsType_tag,&preferencesType_tag,&propertiesType_tag,&restrictionsType_tag,NULL}};
-
-/*+ The RoutinoProfilesType type tag. +*/
-static xmltag RoutinoProfilesType_tag=
-              {"routino-profiles",
-               0, {NULL},
-               NULL,
-               {&profileType_tag,NULL}};
-
-/*+ The xmlDeclaration type tag. +*/
-static xmltag xmlDeclaration_tag=
-              {"xml",
-               2, {"version","encoding"},
-               NULL,
-               {NULL}};
-
-
-/*+ The complete set of tags at the top level. +*/
-static xmltag *xml_toplevel_tags[]={&xmlDeclaration_tag,&RoutinoProfilesType_tag,NULL};
-
 
 /* The XML tag processing functions */
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the speedType XSD type is seen
+  The function that is called when the XML declaration is seen
 
-  int speedType_function Returns 0 if no error occured or something else otherwise.
+  int xmlDeclaration_function Returns 0 if no error occured or something else otherwise.
 
   const char *_tag_ Set to the name of the element tag that triggered this function call.
 
   int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
 
-  const char *highway The contents of the 'highway' attribute (or NULL if not defined).
+  const char *version The contents of the 'version' attribute (or NULL if not defined).
 
-  const char *kph The contents of the 'kph' attribute (or NULL if not defined).
+  const char *encoding The contents of the 'encoding' attribute (or NULL if not defined).
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int speedType_function(const char *_tag_,int _type_,const char *highway,const char *kph)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    double speed;
-    Highway highwaytype;
-
-    XMLPARSE_ASSERT_STRING(_tag_,highway);
-
-    highwaytype=HighwayType(highway);
-
-    if(highwaytype==Way_Count)
-       XMLPARSE_INVALID(_tag_,highway);
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,kph,speed);
-
-    loaded_profiles[nloaded_profiles-1]->speed[highwaytype]=kph_to_speed(speed);
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the speedsType XSD type is seen
-
-  int speedsType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-//static int speedsType_function(const char *_tag_,int _type_)
+//static int xmlDeclaration_function(const char *_tag_,int _type_,const char *version,const char *encoding)
 //{
 // return(0);
 //}
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the preferenceType XSD type is seen
+  The function that is called when the RoutinoProfilesType XSD type is seen
 
-  int preferenceType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *highway The contents of the 'highway' attribute (or NULL if not defined).
-
-  const char *percent The contents of the 'percent' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int preferenceType_function(const char *_tag_,int _type_,const char *highway,const char *percent)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    Highway highwaytype;
-    double p;
-
-    XMLPARSE_ASSERT_STRING(_tag_,highway);
-
-    highwaytype=HighwayType(highway);
-
-    if(highwaytype==Way_Count)
-       XMLPARSE_INVALID(_tag_,highway);
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,percent,p);
-
-    loaded_profiles[nloaded_profiles-1]->highway[highwaytype]=p;
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the preferencesType XSD type is seen
-
-  int preferencesType_function Returns 0 if no error occured or something else otherwise.
+  int RoutinoProfilesType_function Returns 0 if no error occured or something else otherwise.
 
   const char *_tag_ Set to the name of the element tag that triggered this function call.
 
   int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
   ++++++++++++++++++++++++++++++++++++++*/
 
-//static int preferencesType_function(const char *_tag_,int _type_)
-//{
-// return(0);
-//}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the propertyType XSD type is seen
-
-  int propertyType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *type The contents of the 'type' attribute (or NULL if not defined).
-
-  const char *percent The contents of the 'percent' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int propertyType_function(const char *_tag_,int _type_,const char *type,const char *percent)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    Property property;
-    double p;
-
-    XMLPARSE_ASSERT_STRING(_tag_,type);
-
-    property=PropertyType(type);
-
-    if(property==Property_Count)
-       XMLPARSE_INVALID(_tag_,type);
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,percent,p);
-
-    loaded_profiles[nloaded_profiles-1]->props_yes[property]=p;
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the onewayType XSD type is seen
-
-  int onewayType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *obey The contents of the 'obey' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int onewayType_function(const char *_tag_,int _type_,const char *obey)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    int o;
-
-    XMLPARSE_ASSERT_INTEGER(_tag_,obey,o);
-
-    loaded_profiles[nloaded_profiles-1]->oneway=!!o;
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the propertiesType XSD type is seen
-
-  int propertiesType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-//static int propertiesType_function(const char *_tag_,int _type_)
-//{
-// return(0);
-//}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the turnsType XSD type is seen
-
-  int turnsType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *obey The contents of the 'obey' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int turnsType_function(const char *_tag_,int _type_,const char *obey)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    int o;
-
-    XMLPARSE_ASSERT_INTEGER(_tag_,obey,o);
-
-    loaded_profiles[nloaded_profiles-1]->turns=!!o;
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the weightType XSD type is seen
-
-  int weightType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int weightType_function(const char *_tag_,int _type_,const char *limit)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    double l;
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,limit,l);
-
-    loaded_profiles[nloaded_profiles-1]->weight=tonnes_to_weight(l);
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the heightType XSD type is seen
-
-  int heightType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int heightType_function(const char *_tag_,int _type_,const char *limit)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    double l;
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,limit,l);
-
-    loaded_profiles[nloaded_profiles-1]->height=metres_to_height(l);
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the widthType XSD type is seen
-
-  int widthType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int widthType_function(const char *_tag_,int _type_,const char *limit)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    double l;
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,limit,l);
-
-    loaded_profiles[nloaded_profiles-1]->width=metres_to_width(l);
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the lengthType XSD type is seen
-
-  int lengthType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-
-  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int lengthType_function(const char *_tag_,int _type_,const char *limit)
-{
- if(_type_&XMLPARSE_TAG_START)
-   {
-    double l;
-
-    XMLPARSE_ASSERT_FLOATING(_tag_,limit,l);
-
-    loaded_profiles[nloaded_profiles-1]->length=metres_to_length(l);
-   }
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the restrictionsType XSD type is seen
-
-  int restrictionsType_function Returns 0 if no error occured or something else otherwise.
-
-  const char *_tag_ Set to the name of the element tag that triggered this function call.
-
-  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-//static int restrictionsType_function(const char *_tag_,int _type_)
+//static int RoutinoProfilesType_function(const char *_tag_,int _type_)
 //{
 // return(0);
 //}
@@ -571,39 +288,340 @@ static int profileType_function(const char *_tag_,int _type_,const char *name,co
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the RoutinoProfilesType XSD type is seen
+  The function that is called when the speedsType XSD type is seen
 
-  int RoutinoProfilesType_function Returns 0 if no error occured or something else otherwise.
+  int speedsType_function Returns 0 if no error occured or something else otherwise.
 
   const char *_tag_ Set to the name of the element tag that triggered this function call.
 
   int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
   ++++++++++++++++++++++++++++++++++++++*/
 
-//static int RoutinoProfilesType_function(const char *_tag_,int _type_)
+//static int speedsType_function(const char *_tag_,int _type_)
 //{
 // return(0);
 //}
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  The function that is called when the XML declaration is seen
+  The function that is called when the preferencesType XSD type is seen
 
-  int xmlDeclaration_function Returns 0 if no error occured or something else otherwise.
+  int preferencesType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+//static int preferencesType_function(const char *_tag_,int _type_)
+//{
+// return(0);
+//}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the propertiesType XSD type is seen
+
+  int propertiesType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+//static int propertiesType_function(const char *_tag_,int _type_)
+//{
+// return(0);
+//}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the restrictionsType XSD type is seen
+
+  int restrictionsType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+//static int restrictionsType_function(const char *_tag_,int _type_)
+//{
+// return(0);
+//}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the speedType XSD type is seen
+
+  int speedType_function Returns 0 if no error occured or something else otherwise.
 
   const char *_tag_ Set to the name of the element tag that triggered this function call.
 
   int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
 
-  const char *version The contents of the 'version' attribute (or NULL if not defined).
+  const char *highway The contents of the 'highway' attribute (or NULL if not defined).
 
-  const char *encoding The contents of the 'encoding' attribute (or NULL if not defined).
+  const char *kph The contents of the 'kph' attribute (or NULL if not defined).
   ++++++++++++++++++++++++++++++++++++++*/
 
-//static int xmlDeclaration_function(const char *_tag_,int _type_,const char *version,const char *encoding)
-//{
-// return(0);
-//}
+static int speedType_function(const char *_tag_,int _type_,const char *highway,const char *kph)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    double speed;
+    Highway highwaytype;
+
+    XMLPARSE_ASSERT_STRING(_tag_,highway);
+
+    highwaytype=HighwayType(highway);
+
+    if(highwaytype==Highway_None)
+       XMLPARSE_INVALID(_tag_,highway);
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,kph); speed=atof(kph);
+
+    loaded_profiles[nloaded_profiles-1]->speed[highwaytype]=kph_to_speed(speed);
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the preferenceType XSD type is seen
+
+  int preferenceType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *highway The contents of the 'highway' attribute (or NULL if not defined).
+
+  const char *percent The contents of the 'percent' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int preferenceType_function(const char *_tag_,int _type_,const char *highway,const char *percent)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    Highway highwaytype;
+    double p;
+
+    XMLPARSE_ASSERT_STRING(_tag_,highway);
+
+    highwaytype=HighwayType(highway);
+
+    if(highwaytype==Highway_None)
+       XMLPARSE_INVALID(_tag_,highway);
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,percent); p=atof(percent);
+
+    loaded_profiles[nloaded_profiles-1]->highway[highwaytype]=p;
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the propertyType XSD type is seen
+
+  int propertyType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *type The contents of the 'type' attribute (or NULL if not defined).
+
+  const char *percent The contents of the 'percent' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int propertyType_function(const char *_tag_,int _type_,const char *type,const char *percent)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    Property property;
+    double p;
+
+    XMLPARSE_ASSERT_STRING(_tag_,type);
+
+    property=PropertyType(type);
+
+    if(property==Property_None)
+       XMLPARSE_INVALID(_tag_,type);
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,percent); p=atof(percent);
+
+    loaded_profiles[nloaded_profiles-1]->props_yes[property]=p;
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the onewayType XSD type is seen
+
+  int onewayType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *obey The contents of the 'obey' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int onewayType_function(const char *_tag_,int _type_,const char *obey)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    int o;
+
+    XMLPARSE_ASSERT_INTEGER(_tag_,obey); o=atoi(obey);
+
+    loaded_profiles[nloaded_profiles-1]->oneway=!!o;
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the turnsType XSD type is seen
+
+  int turnsType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *obey The contents of the 'obey' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int turnsType_function(const char *_tag_,int _type_,const char *obey)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    int o;
+
+    XMLPARSE_ASSERT_INTEGER(_tag_,obey); o=atoi(obey);
+
+    loaded_profiles[nloaded_profiles-1]->turns=!!o;
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the weightType XSD type is seen
+
+  int weightType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int weightType_function(const char *_tag_,int _type_,const char *limit)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    double l;
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,limit); l=atof(limit);
+
+    loaded_profiles[nloaded_profiles-1]->weight=tonnes_to_weight(l);
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the heightType XSD type is seen
+
+  int heightType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int heightType_function(const char *_tag_,int _type_,const char *limit)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    double l;
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,limit); l=atof(limit);
+
+    loaded_profiles[nloaded_profiles-1]->height=metres_to_height(l);
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the widthType XSD type is seen
+
+  int widthType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int widthType_function(const char *_tag_,int _type_,const char *limit)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    double l;
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,limit); l=atof(limit);
+
+    loaded_profiles[nloaded_profiles-1]->width=metres_to_width(l);
+   }
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the lengthType XSD type is seen
+
+  int lengthType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *limit The contents of the 'limit' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int lengthType_function(const char *_tag_,int _type_,const char *limit)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    double l;
+
+    XMLPARSE_ASSERT_FLOATING(_tag_,limit); l=atof(limit);
+
+    loaded_profiles[nloaded_profiles-1]->length=metres_to_length(l);
+   }
+
+ return(0);
+}
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -616,6 +634,7 @@ static int profileType_function(const char *_tag_,int _type_,const char *name,co
 
 int ParseXMLProfiles(const char *filename)
 {
+ int fd;
  int retval;
 
  if(!ExistsFile(filename))
@@ -624,17 +643,11 @@ int ParseXMLProfiles(const char *filename)
     return(1);
    }
 
- FILE *file=fopen(filename,"r");
+ fd=ReOpenFile(filename);
 
- if(!file)
-   {
-    fprintf(stderr,"Error: Cannot open profiles file '%s' for reading.\n",filename);
-    return(1);
-   }
+ retval=ParseXML(fd,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_ERRNONAME);
 
- retval=ParseXML(file,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_ERRNONAME);
-
- fclose(file);
+ CloseFile(fd);
 
  if(retval)
    {
@@ -697,7 +710,7 @@ int UpdateProfile(Profile *profile,Ways *ways)
 
  /* Normalise the highway preferences into the range ~0 -> 1 */
 
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
    {
     if(profile->highway[i]<0)
        profile->highway[i]=0;
@@ -709,7 +722,7 @@ int UpdateProfile(Profile *profile,Ways *ways)
  if(hmax==0)
     return(1);
 
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
    {
     profile->highway[i]/=hmax;
 
@@ -748,7 +761,7 @@ int UpdateProfile(Profile *profile,Ways *ways)
 
  profile->max_speed=0;
 
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
     if(profile->speed[i]>profile->max_speed)
        profile->max_speed=profile->speed[i];
 
@@ -780,7 +793,7 @@ int UpdateProfile(Profile *profile,Ways *ways)
 
 void PrintProfile(const Profile *profile)
 {
- unsigned int i;
+ int i;
 
  printf("Profile\n=======\n");
 
@@ -790,12 +803,12 @@ void PrintProfile(const Profile *profile)
 
  printf("\n");
 
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
     printf("Highway %-12s: %3d%%\n",HighwayName(i),(int)profile->highway[i]);
 
  printf("\n");
 
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
     if(profile->highway[i])
        printf("Speed on %-12s: %3d km/h / %2.0f mph\n",HighwayName(i),profile->speed[i],(double)profile->speed[i]/1.6);
 
@@ -821,7 +834,7 @@ void PrintProfile(const Profile *profile)
 
 void PrintProfilesXML(void)
 {
- unsigned int i,j;
+ int i,j;
  char *padding="                ";
 
  printf("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
@@ -835,12 +848,12 @@ void PrintProfilesXML(void)
     printf("  <profile name=\"%s\" transport=\"%s\">\n",loaded_profiles[j]->name,TransportName(loaded_profiles[j]->transport));
 
     printf("    <speeds>\n");
-    for(i=1;i<Way_Count;i++)
+    for(i=1;i<Highway_Count;i++)
        printf("      <speed highway=\"%s\"%s kph=\"%d\" />\n",HighwayName(i),padding+3+strlen(HighwayName(i)),loaded_profiles[j]->speed[i]);
     printf("    </speeds>\n");
 
     printf("    <preferences>\n");
-    for(i=1;i<Way_Count;i++)
+    for(i=1;i<Highway_Count;i++)
        printf("      <preference highway=\"%s\"%s percent=\"%.0f\" />\n",HighwayName(i),padding+3+strlen(HighwayName(i)),loaded_profiles[j]->highway[i]);
     printf("    </preferences>\n");
 
@@ -872,7 +885,7 @@ void PrintProfilesXML(void)
 
 void PrintProfilesJSON(void)
 {
- unsigned int i,j;
+ int i,j;
 
  printf("var routino={ // contains all default Routino options (generated using \"--help-profile-json\").\n");
  printf("\n");
@@ -890,7 +903,7 @@ void PrintProfilesJSON(void)
 
  printf("  // Highway types\n");
  printf("  highways: { ");
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
     printf("%s%s: %d",i==1?"":", ",HighwayName(i),i);
  printf(" },\n");
  printf("\n");
@@ -908,24 +921,24 @@ void PrintProfilesJSON(void)
 
  printf("  // Allowed highways\n");
  printf("  profile_highway: {\n");
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
    {
     printf("    %12s: { ",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
        printf("%s%s: %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->highway[i]);
-    printf(" }%s\n",i==(Way_Count-1)?"":",");
+    printf(" }%s\n",i==(Highway_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
 
  printf("  // Speed limits\n");
  printf("  profile_speed: {\n");
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
    {
     printf("    %12s: { ",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
        printf("%s%s: %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->speed[i]);
-    printf(" }%s\n",i==(Way_Count-1)?"":",");
+    printf(" }%s\n",i==(Highway_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
@@ -981,7 +994,7 @@ void PrintProfilesJSON(void)
 
 void PrintProfilesPerl(void)
 {
- unsigned int i,j;
+ int i,j;
 
  printf("$routino={ # contains all default Routino options (generated using \"--help-profile-perl\").\n");
  printf("\n");
@@ -999,7 +1012,7 @@ void PrintProfilesPerl(void)
 
  printf("  # Highway types\n");
  printf("  highways => { ");
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
     printf("%s%s => %d",i==1?"":", ",HighwayName(i),i);
  printf(" },\n");
  printf("\n");
@@ -1017,24 +1030,24 @@ void PrintProfilesPerl(void)
 
  printf("  # Allowed highways\n");
  printf("  profile_highway => {\n");
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
    {
     printf("  %12s => {",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
        printf("%s %s => %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->highway[i]);
-    printf(" }%s\n",i==(Way_Count-1)?"":",");
+    printf(" }%s\n",i==(Highway_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
 
  printf("  # Speed limits\n");
  printf("  profile_speed => {\n");
- for(i=1;i<Way_Count;i++)
+ for(i=1;i<Highway_Count;i++)
    {
     printf("  %12s => {",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
        printf("%s %s => %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->speed[i]);
-    printf(" }%s\n",i==(Way_Count-1)?"":",");
+    printf(" }%s\n",i==(Highway_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
