@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/types.c,v 1.6 2010/09/17 17:43:41 amb Exp $
-
  Functions for handling the data types.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -40,47 +38,47 @@ Highway HighwayType(const char *highway)
  switch(*highway)
    {
    case 'c':
-    if(!strcmp(highway,"cycleway")) return(Way_Cycleway);
-    return(Way_Count);
+    if(!strcmp(highway,"cycleway")) return(Highway_Cycleway);
+    break;
 
    case 'f':
-    if(!strcmp(highway,"ferry")) return(Way_Ferry);
-    return(Way_Count);
+    if(!strcmp(highway,"ferry")) return(Highway_Ferry);
+    break;
 
    case 'm':
-    if(!strcmp(highway,"motorway")) return(Way_Motorway);
-    return(Way_Count);
+    if(!strcmp(highway,"motorway")) return(Highway_Motorway);
+    break;
 
    case 'p':
-    if(!strcmp(highway,"primary")) return(Way_Primary);
-    if(!strcmp(highway,"path")) return(Way_Path);
-    return(Way_Count);
+    if(!strcmp(highway,"primary")) return(Highway_Primary);
+    if(!strcmp(highway,"path")) return(Highway_Path);
+    break;
 
    case 'r':
-    if(!strcmp(highway,"residential")) return(Way_Residential);
-    return(Way_Count);
+    if(!strcmp(highway,"residential")) return(Highway_Residential);
+    break;
 
    case 's':
-    if(!strcmp(highway,"secondary")) return(Way_Secondary);
-    if(!strcmp(highway,"service")) return(Way_Service);
-    if(!strcmp(highway,"steps")) return(Way_Steps);
-    return(Way_Count);
+    if(!strcmp(highway,"secondary")) return(Highway_Secondary);
+    if(!strcmp(highway,"service")) return(Highway_Service);
+    if(!strcmp(highway,"steps")) return(Highway_Steps);
+    break;
 
    case 't':
-    if(!strcmp(highway,"trunk")) return(Way_Trunk);
-    if(!strcmp(highway,"tertiary")) return(Way_Tertiary);
-    if(!strcmp(highway,"track")) return(Way_Track);
-    return(Way_Count);
+    if(!strcmp(highway,"trunk")) return(Highway_Trunk);
+    if(!strcmp(highway,"tertiary")) return(Highway_Tertiary);
+    if(!strcmp(highway,"track")) return(Highway_Track);
+    break;
 
    case 'u':
-    if(!strcmp(highway,"unclassified")) return(Way_Unclassified);
-    return(Way_Count);
+    if(!strcmp(highway,"unclassified")) return(Highway_Unclassified);
+    break;
 
    default:
     ;
    }
 
- return(Way_Count);
+ return(Highway_None);
 }
 
 
@@ -121,8 +119,8 @@ Transport TransportType(const char *transport)
    case 'm':
     if(!strcmp(transport,"moped"))
        return(Transport_Moped);
-    if(!strcmp(transport,"motorbike"))
-       return(Transport_Motorbike);
+    if(!strcmp(transport,"motorcycle"))
+       return(Transport_Motorcycle);
     if(!strcmp(transport,"motorcar"))
        return(Transport_Motorcar);
     break;
@@ -205,38 +203,42 @@ const char *HighwayName(Highway highway)
 {
  switch(highway)
    {
-   case Way_Motorway:
+   case Highway_None:
+    return("NONE");
+
+   case Highway_Motorway:
     return("motorway");
-   case Way_Trunk:
+   case Highway_Trunk:
     return("trunk");
-   case Way_Primary:
+   case Highway_Primary:
     return("primary");
-   case Way_Secondary:
+   case Highway_Secondary:
     return("secondary");
-   case Way_Tertiary:
+   case Highway_Tertiary:
     return("tertiary");
-   case Way_Unclassified:
+   case Highway_Unclassified:
     return("unclassified");
-   case Way_Residential:
+   case Highway_Residential:
     return("residential");
-   case Way_Service:
+   case Highway_Service:
     return("service");
-   case Way_Track:
+   case Highway_Track:
     return("track");
-   case Way_Cycleway:
+   case Highway_Cycleway:
     return("cycleway");
-   case Way_Path:
+   case Highway_Path:
     return("path");
-   case Way_Steps:
+   case Highway_Steps:
     return("steps");
-   case Way_Ferry:
+   case Highway_Ferry:
     return("ferry");
 
-   case Way_Count:
+   case Highway_Count:
     ;
 
-   case Way_OneWay:
-   case Way_Roundabout:
+   case Highway_OneWay:
+    ;
+   case Highway_Roundabout:
     ;
    }
 
@@ -269,8 +271,8 @@ const char *TransportName(Transport transport)
     return("bicycle");
    case Transport_Moped:
     return("moped");
-   case Transport_Motorbike:
-    return("motorbike");
+   case Transport_Motorcycle:
+    return("motorcycle");
    case Transport_Motorcar:
     return("motorcar");
    case Transport_Goods:
@@ -330,71 +332,158 @@ const char *PropertyName(Property property)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  A string containing the names of allowed transports on a way.
+  A string containing the names of highways.
 
-  const char *AllowedNameList Returns the list of names.
+  const char *HighwaysNameList Returns the list of names.
 
-  allow_t allowed The allowed type.
+  highways_t highways The highways type.
   ++++++++++++++++++++++++++++++++++++++*/
 
-const char *AllowedNameList(allow_t allowed)
+const char *HighwaysNameList(highways_t highways)
 {
  static char string[256];
 
  string[0]=0;
 
- if(allowed & Allow_Foot)
+ if(highways & Highways_Motorway)
+    strcat(string,"motorway");
+
+ if(highways & Highways_Trunk)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"trunk");
+   }
+
+ if(highways & Highways_Primary)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"primary");
+   }
+
+ if(highways & Highways_Tertiary)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"tertiary");
+   }
+
+ if(highways & Highways_Unclassified)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"unclassified");
+   }
+
+ if(highways & Highways_Residential)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"residential");
+   }
+
+ if(highways & Highways_Service)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"service");
+   }
+
+ if(highways & Highways_Track)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"track");
+   }
+
+ if(highways & Highways_Cycleway)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"cycleway");
+   }
+
+ if(highways & Highways_Path)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"path");
+   }
+
+ if(highways & Highways_Steps)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"steps");
+   }
+
+ if(highways & Highways_Ferry)
+   {
+    if(*string) strcat(string,", ");
+    strcat(string,"ferry");
+   }
+
+ return(string);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  A string containing the names of allowed transports on a way.
+
+  const char *AllowedNameList Returns the list of names.
+
+  transports_t allowed The allowed type.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+const char *AllowedNameList(transports_t allowed)
+{
+ static char string[256];
+
+ string[0]=0;
+
+ if(allowed & Transports_Foot)
     strcat(string,"foot");
 
- if(allowed & Allow_Horse)
+ if(allowed & Transports_Horse)
    {
     if(*string) strcat(string,", ");
     strcat(string,"horse");
    }
 
- if(allowed & Allow_Wheelchair)
+ if(allowed & Transports_Wheelchair)
    {
     if(*string) strcat(string,", ");
     strcat(string,"wheelchair");
    }
 
- if(allowed & Allow_Bicycle)
+ if(allowed & Transports_Bicycle)
    {
     if(*string) strcat(string,", ");
     strcat(string,"bicycle");
    }
 
- if(allowed & Allow_Moped)
+ if(allowed & Transports_Moped)
    {
     if(*string) strcat(string,", ");
     strcat(string,"moped");
    }
 
- if(allowed & Allow_Motorbike)
+ if(allowed & Transports_Motorcycle)
    {
     if(*string) strcat(string,", ");
-    strcat(string,"motorbike");
+    strcat(string,"motorcycle");
    }
 
- if(allowed & Allow_Motorcar)
+ if(allowed & Transports_Motorcar)
    {
     if(*string) strcat(string,", ");
     strcat(string,"motorcar");
    }
 
- if(allowed & Allow_Goods)
+ if(allowed & Transports_Goods)
    {
     if(*string) strcat(string,", ");
     strcat(string,"goods");
    }
 
- if(allowed & Allow_HGV)
+ if(allowed & Transports_HGV)
    {
     if(*string) strcat(string,", ");
     strcat(string,"hgv");
    }
 
- if(allowed & Allow_PSV)
+ if(allowed & Transports_PSV)
    {
     if(*string) strcat(string,", ");
     strcat(string,"psv");
@@ -409,10 +498,10 @@ const char *AllowedNameList(allow_t allowed)
 
   const char *PropertiesNameList Returns the list of names.
 
-  wayprop_t properties The properties of the way.
+  properties_t properties The properties of the way.
   ++++++++++++++++++++++++++++++++++++++*/
 
-const char *PropertiesNameList(wayprop_t properties)
+const char *PropertiesNameList(properties_t properties)
 {
  static char string[256];
 
@@ -495,8 +584,8 @@ const char *TransportList(void)
         "    bicycle    = Bicycle\n"
         "    wheelchair = Wheelchair\n"
         "    horse      = Horse\n"
-        "    moped      = Moped     (Small motorbike, limited speed)\n"
-        "    motorbike  = Motorbike\n"
+        "    moped      = Moped     (Small motorcycle, limited speed)\n"
+        "    motorcycle = Motorcycle\n"
         "    motorcar   = Motorcar\n"
         "    goods      = Goods     (Small lorry, van)\n"
         "    hgv        = HGV       (Heavy Goods Vehicle - large lorry)\n"
