@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -40,16 +40,16 @@
 
 
 /*+ An undefined node index. +*/
-#define NO_NODE        (~(index_t)0)
+#define NO_NODE        ((index_t)~0)
 
 /*+ An undefined segment index. +*/
-#define NO_SEGMENT     (~(index_t)0)
+#define NO_SEGMENT     ((index_t)~0)
 
 /*+ An undefined way index. +*/
-#define NO_WAY         (~(index_t)0)
+#define NO_WAY         ((index_t)~0)
 
 /*+ An undefined relation index. +*/
-#define NO_RELATION    (~(index_t)0)
+#define NO_RELATION    ((index_t)~0)
 
 /*+ An undefined location. +*/
 #define NO_LATLONG     ((latlong_t)0x80000000)
@@ -68,39 +68,46 @@
 /*+ The latitude and longitude integer range within each bin. +*/
 #define LAT_LONG_BIN   65536
 
+
 /*+ A flag to mark a node as a super-node. +*/
-#define NODE_SUPER     ((uint16_t)0x8000)
+#define NODE_SUPER       ((nodeflags_t)0x8000)
 
 /*+ A flag to mark a node as suitable for a U-turn. +*/
-#define NODE_UTURN     ((uint16_t)0x4000)
+#define NODE_UTURN       ((nodeflags_t)0x4000)
 
 /*+ A flag to mark a node as a mini-roundabout. +*/
-#define NODE_MINIRNDBT ((uint16_t)0x2000)
+#define NODE_MINIRNDBT   ((nodeflags_t)0x2000)
 
 /*+ A flag to mark a node as a turn relation via node. +*/
-#define NODE_TURNRSTRCT ((uint16_t)0x1000)
+#define NODE_TURNRSTRCT  ((nodeflags_t)0x1000)
 
-/*+ A flag to mark a node as a turn relation via node. +*/
-#define NODE_TURNRSTRCT2 ((uint16_t)0x0800)
+/*+ A flag to mark a node as adjacent to a turn relation via node. +*/
+#define NODE_TURNRSTRCT2 ((nodeflags_t)0x0800)
 
+/*+ A flag to mark a node as deleted. +*/
+#define NODE_DELETED     ((nodeflags_t)0x0400)
+
+
+/*+ A flag to mark a segment as being part of an area (must be the highest valued flag). +*/
+#define SEGMENT_AREA   ((distance_t)0x80000000)
 
 /*+ A flag to mark a segment as one-way from node1 to node2. +*/
-#define ONEWAY_1TO2    ((distance_t)0x80000000)
+#define ONEWAY_1TO2    ((distance_t)0x40000000)
 
 /*+ A flag to mark a segment as one-way from node2 to node1. +*/
-#define ONEWAY_2TO1    ((distance_t)0x40000000)
+#define ONEWAY_2TO1    ((distance_t)0x20000000)
 
 /*+ A flag to mark a segment as a super-segment. +*/
-#define SEGMENT_SUPER  ((distance_t)0x20000000)
+#define SEGMENT_SUPER  ((distance_t)0x10000000)
 
 /*+ A flag to mark a segment as a normal segment. +*/
-#define SEGMENT_NORMAL ((distance_t)0x10000000)
+#define SEGMENT_NORMAL ((distance_t)0x08000000)
 
 /*+ The real distance ignoring the other flags. +*/
-#define DISTANCE(xx)   ((distance_t)((xx)&(~(ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL))))
+#define DISTANCE(xx)   ((distance_t)((xx)&(~(SEGMENT_AREA|ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL))))
 
 /*+ The distance flags selecting only the flags. +*/
-#define DISTFLAG(xx)   ((distance_t)((xx)&(ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL)))
+#define DISTFLAG(xx)   ((distance_t)((xx)&(SEGMENT_AREA|ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL)))
 
 
 /*+ A very large almost infinite distance. +*/
@@ -110,6 +117,14 @@
 #define INF_SCORE      (score_t)1E30
 
 
+/*+ A flag to mark a deleted way. +*/
+#define WAY_DELETED ((highway_t)~0)
+
+
+/*+ A flag to mark a deleted relation. +*/
+#define RELATION_DELETED ((transports_t)~0)
+
+
 /* Simple Types */
 
 
@@ -161,6 +176,9 @@ typedef uint16_t ll_off_t;
 #define degrees_to_radians(xxx) ((xxx)*(M_PI/180.0))
 
 
+/*+ Node flags. +*/
+typedef uint16_t nodeflags_t;
+
 /*+ A distance, measured in metres. +*/
 typedef uint32_t distance_t;
 
@@ -196,24 +214,26 @@ typedef uint8_t highway_t;
 /*+ The different types of a highway. +*/
 typedef enum _Highway
  {
-  Way_Motorway    = 1,
-  Way_Trunk       = 2,
-  Way_Primary     = 3,
-  Way_Secondary   = 4,
-  Way_Tertiary    = 5,
-  Way_Unclassified= 6,
-  Way_Residential = 7,
-  Way_Service     = 8,
-  Way_Track       = 9,
-  Way_Cycleway    =10,
-  Way_Path        =11,
-  Way_Steps       =12,
-  Way_Ferry       =13,
+  Highway_None         =  0,
 
-  Way_Count       =14,       /* One more than the number of highway types. */
+  Highway_Motorway     =  1,
+  Highway_Trunk        =  2,
+  Highway_Primary      =  3,
+  Highway_Secondary    =  4,
+  Highway_Tertiary     =  5,
+  Highway_Unclassified =  6,
+  Highway_Residential  =  7,
+  Highway_Service      =  8,
+  Highway_Track        =  9,
+  Highway_Cycleway     = 10,
+  Highway_Path         = 11,
+  Highway_Steps        = 12,
+  Highway_Ferry        = 13,
 
-  Way_OneWay      =32,
-  Way_Roundabout  =64
+  Highway_Count        = 14,       /* One more than the number of highway types. */
+
+  Highway_OneWay       = 32,
+  Highway_Roundabout   = 64
  }
  Highway;
 
@@ -229,19 +249,19 @@ typedef enum _Highways
  {
   Highways_None         = 0,
 
-  Highways_Motorway     = HIGHWAYS(Way_Motorway    ),
-  Highways_Trunk        = HIGHWAYS(Way_Trunk       ),
-  Highways_Primary      = HIGHWAYS(Way_Primary     ),
-  Highways_Secondary    = HIGHWAYS(Way_Secondary   ),
-  Highways_Tertiary     = HIGHWAYS(Way_Tertiary    ),
-  Highways_Unclassified = HIGHWAYS(Way_Unclassified),
-  Highways_Residential  = HIGHWAYS(Way_Residential ),
-  Highways_Service      = HIGHWAYS(Way_Service     ),
-  Highways_Track        = HIGHWAYS(Way_Track       ),
-  Highways_Cycleway     = HIGHWAYS(Way_Cycleway    ),
-  Highways_Path         = HIGHWAYS(Way_Path        ),
-  Highways_Steps        = HIGHWAYS(Way_Steps       ),
-  Highways_Ferry        = HIGHWAYS(Way_Ferry       )
+  Highways_Motorway     = HIGHWAYS(Highway_Motorway    ),
+  Highways_Trunk        = HIGHWAYS(Highway_Trunk       ),
+  Highways_Primary      = HIGHWAYS(Highway_Primary     ),
+  Highways_Secondary    = HIGHWAYS(Highway_Secondary   ),
+  Highways_Tertiary     = HIGHWAYS(Highway_Tertiary    ),
+  Highways_Unclassified = HIGHWAYS(Highway_Unclassified),
+  Highways_Residential  = HIGHWAYS(Highway_Residential ),
+  Highways_Service      = HIGHWAYS(Highway_Service     ),
+  Highways_Track        = HIGHWAYS(Highway_Track       ),
+  Highways_Cycleway     = HIGHWAYS(Highway_Cycleway    ),
+  Highways_Path         = HIGHWAYS(Highway_Path        ),
+  Highways_Steps        = HIGHWAYS(Highway_Steps       ),
+  Highways_Ferry        = HIGHWAYS(Highway_Ferry       )
  }
  Highways;
 
@@ -259,7 +279,7 @@ typedef enum _Transport
   Transport_Wheelchair =  3,
   Transport_Bicycle    =  4,
   Transport_Moped      =  5,
-  Transport_Motorbike  =  6,
+  Transport_Motorcycle =  6,
   Transport_Motorcar   =  7,
   Transport_Goods      =  8,
   Transport_HGV        =  9,
@@ -285,7 +305,7 @@ typedef enum _Transports
   Transports_Wheelchair = TRANSPORTS(Transport_Wheelchair),
   Transports_Bicycle    = TRANSPORTS(Transport_Bicycle   ),
   Transports_Moped      = TRANSPORTS(Transport_Moped     ),
-  Transports_Motorbike  = TRANSPORTS(Transport_Motorbike ),
+  Transports_Motorcycle = TRANSPORTS(Transport_Motorcycle),
   Transports_Motorcar   = TRANSPORTS(Transport_Motorcar  ),
   Transports_Goods      = TRANSPORTS(Transport_Goods     ),
   Transports_HGV        = TRANSPORTS(Transport_HGV       ),
